
{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)
-}

module GI.Gio.Objects.ConverterInputStream
    ( 

-- * Exported types
    ConverterInputStream(..)                ,
    ConverterInputStreamK                   ,
    toConverterInputStream                  ,
    noConverterInputStream                  ,


 -- * Methods
-- ** converterInputStreamGetConverter
    converterInputStreamGetConverter        ,


-- ** converterInputStreamNew
    converterInputStreamNew                 ,




 -- * Properties
-- ** Converter
    ConverterInputStreamConverterPropertyInfo,
    constructConverterInputStreamConverter  ,
    getConverterInputStreamConverter        ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gio.Types
import GI.Gio.Callbacks
import qualified GI.GObject as GObject

newtype ConverterInputStream = ConverterInputStream (ForeignPtr ConverterInputStream)
foreign import ccall "g_converter_input_stream_get_type"
    c_g_converter_input_stream_get_type :: IO GType

type instance ParentTypes ConverterInputStream = ConverterInputStreamParentTypes
type ConverterInputStreamParentTypes = '[FilterInputStream, InputStream, GObject.Object, PollableInputStream]

instance GObject ConverterInputStream where
    gobjectIsInitiallyUnowned _ = False
    gobjectType _ = c_g_converter_input_stream_get_type
    

class GObject o => ConverterInputStreamK o
instance (GObject o, IsDescendantOf ConverterInputStream o) => ConverterInputStreamK o

toConverterInputStream :: ConverterInputStreamK o => o -> IO ConverterInputStream
toConverterInputStream = unsafeCastTo ConverterInputStream

noConverterInputStream :: Maybe ConverterInputStream
noConverterInputStream = Nothing

-- VVV Prop "converter"
   -- Type: TInterface "Gio" "Converter"
   -- Flags: [PropertyReadable,PropertyWritable,PropertyConstructOnly]

getConverterInputStreamConverter :: (MonadIO m, ConverterInputStreamK o) => o -> m Converter
getConverterInputStreamConverter obj = liftIO $ getObjectPropertyObject obj "converter" Converter

constructConverterInputStreamConverter :: (ConverterK a) => a -> IO ([Char], GValue)
constructConverterInputStreamConverter val = constructObjectPropertyObject "converter" val

data ConverterInputStreamConverterPropertyInfo
instance AttrInfo ConverterInputStreamConverterPropertyInfo where
    type AttrAllowedOps ConverterInputStreamConverterPropertyInfo = '[ 'AttrConstruct, 'AttrGet]
    type AttrSetTypeConstraint ConverterInputStreamConverterPropertyInfo = ConverterK
    type AttrBaseTypeConstraint ConverterInputStreamConverterPropertyInfo = ConverterInputStreamK
    type AttrGetType ConverterInputStreamConverterPropertyInfo = Converter
    type AttrLabel ConverterInputStreamConverterPropertyInfo = "ConverterInputStream::converter"
    attrGet _ = getConverterInputStreamConverter
    attrSet _ = undefined
    attrConstruct _ = constructConverterInputStreamConverter

type instance AttributeList ConverterInputStream = ConverterInputStreamAttributeList
type ConverterInputStreamAttributeList = ('[ '("base-stream", FilterInputStreamBaseStreamPropertyInfo), '("close-base-stream", FilterInputStreamCloseBaseStreamPropertyInfo), '("converter", ConverterInputStreamConverterPropertyInfo)] :: [(Symbol, *)])

type instance SignalList ConverterInputStream = ConverterInputStreamSignalList
type ConverterInputStreamSignalList = ('[ '("notify", GObject.ObjectNotifySignalInfo), '("notify::[property]", GObjectNotifySignalInfo)] :: [(Symbol, *)])

-- method ConverterInputStream::new
-- method type : Constructor
-- Args : [Arg {argName = "base_stream", argType = TInterface "Gio" "InputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing},Arg {argName = "converter", argType = TInterface "Gio" "Converter", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "base_stream", argType = TInterface "Gio" "InputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing},Arg {argName = "converter", argType = TInterface "Gio" "Converter", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "ConverterInputStream"
-- throws : False
-- Skip return : False

foreign import ccall "g_converter_input_stream_new" g_converter_input_stream_new :: 
    Ptr InputStream ->                      -- base_stream : TInterface "Gio" "InputStream"
    Ptr Converter ->                        -- converter : TInterface "Gio" "Converter"
    IO (Ptr ConverterInputStream)


converterInputStreamNew ::
    (MonadIO m, InputStreamK a, ConverterK b) =>
    a ->                                    -- base_stream
    b ->                                    -- converter
    m ConverterInputStream
converterInputStreamNew base_stream converter = liftIO $ do
    let base_stream' = unsafeManagedPtrCastPtr base_stream
    let converter' = unsafeManagedPtrCastPtr converter
    result <- g_converter_input_stream_new base_stream' converter'
    checkUnexpectedReturnNULL "g_converter_input_stream_new" result
    result' <- (wrapObject ConverterInputStream) result
    touchManagedPtr base_stream
    touchManagedPtr converter
    return result'

-- method ConverterInputStream::get_converter
-- method type : OrdinaryMethod
-- Args : [Arg {argName = "_obj", argType = TInterface "Gio" "ConverterInputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "_obj", argType = TInterface "Gio" "ConverterInputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "Converter"
-- throws : False
-- Skip return : False

foreign import ccall "g_converter_input_stream_get_converter" g_converter_input_stream_get_converter :: 
    Ptr ConverterInputStream ->             -- _obj : TInterface "Gio" "ConverterInputStream"
    IO (Ptr Converter)


converterInputStreamGetConverter ::
    (MonadIO m, ConverterInputStreamK a) =>
    a ->                                    -- _obj
    m Converter
converterInputStreamGetConverter _obj = liftIO $ do
    let _obj' = unsafeManagedPtrCastPtr _obj
    result <- g_converter_input_stream_get_converter _obj'
    checkUnexpectedReturnNULL "g_converter_input_stream_get_converter" result
    result' <- (newObject Converter) result
    touchManagedPtr _obj
    return result'


