
{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)
-}

module GI.Gio.Objects.ConverterOutputStream
    ( 

-- * Exported types
    ConverterOutputStream(..)               ,
    ConverterOutputStreamK                  ,
    toConverterOutputStream                 ,
    noConverterOutputStream                 ,


 -- * Methods
-- ** converterOutputStreamGetConverter
    converterOutputStreamGetConverter       ,


-- ** converterOutputStreamNew
    converterOutputStreamNew                ,




 -- * Properties
-- ** Converter
    ConverterOutputStreamConverterPropertyInfo,
    constructConverterOutputStreamConverter ,
    getConverterOutputStreamConverter       ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gio.Types
import GI.Gio.Callbacks
import qualified GI.GObject as GObject

newtype ConverterOutputStream = ConverterOutputStream (ForeignPtr ConverterOutputStream)
foreign import ccall "g_converter_output_stream_get_type"
    c_g_converter_output_stream_get_type :: IO GType

type instance ParentTypes ConverterOutputStream = ConverterOutputStreamParentTypes
type ConverterOutputStreamParentTypes = '[FilterOutputStream, OutputStream, GObject.Object, PollableOutputStream]

instance GObject ConverterOutputStream where
    gobjectIsInitiallyUnowned _ = False
    gobjectType _ = c_g_converter_output_stream_get_type
    

class GObject o => ConverterOutputStreamK o
instance (GObject o, IsDescendantOf ConverterOutputStream o) => ConverterOutputStreamK o

toConverterOutputStream :: ConverterOutputStreamK o => o -> IO ConverterOutputStream
toConverterOutputStream = unsafeCastTo ConverterOutputStream

noConverterOutputStream :: Maybe ConverterOutputStream
noConverterOutputStream = Nothing

-- VVV Prop "converter"
   -- Type: TInterface "Gio" "Converter"
   -- Flags: [PropertyReadable,PropertyWritable,PropertyConstructOnly]

getConverterOutputStreamConverter :: (MonadIO m, ConverterOutputStreamK o) => o -> m Converter
getConverterOutputStreamConverter obj = liftIO $ getObjectPropertyObject obj "converter" Converter

constructConverterOutputStreamConverter :: (ConverterK a) => a -> IO ([Char], GValue)
constructConverterOutputStreamConverter val = constructObjectPropertyObject "converter" val

data ConverterOutputStreamConverterPropertyInfo
instance AttrInfo ConverterOutputStreamConverterPropertyInfo where
    type AttrAllowedOps ConverterOutputStreamConverterPropertyInfo = '[ 'AttrConstruct, 'AttrGet]
    type AttrSetTypeConstraint ConverterOutputStreamConverterPropertyInfo = ConverterK
    type AttrBaseTypeConstraint ConverterOutputStreamConverterPropertyInfo = ConverterOutputStreamK
    type AttrGetType ConverterOutputStreamConverterPropertyInfo = Converter
    type AttrLabel ConverterOutputStreamConverterPropertyInfo = "ConverterOutputStream::converter"
    attrGet _ = getConverterOutputStreamConverter
    attrSet _ = undefined
    attrConstruct _ = constructConverterOutputStreamConverter

type instance AttributeList ConverterOutputStream = ConverterOutputStreamAttributeList
type ConverterOutputStreamAttributeList = ('[ '("base-stream", FilterOutputStreamBaseStreamPropertyInfo), '("close-base-stream", FilterOutputStreamCloseBaseStreamPropertyInfo), '("converter", ConverterOutputStreamConverterPropertyInfo)] :: [(Symbol, *)])

type instance SignalList ConverterOutputStream = ConverterOutputStreamSignalList
type ConverterOutputStreamSignalList = ('[ '("notify", GObject.ObjectNotifySignalInfo), '("notify::[property]", GObjectNotifySignalInfo)] :: [(Symbol, *)])

-- method ConverterOutputStream::new
-- method type : Constructor
-- Args : [Arg {argName = "base_stream", argType = TInterface "Gio" "OutputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing},Arg {argName = "converter", argType = TInterface "Gio" "Converter", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "base_stream", argType = TInterface "Gio" "OutputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing},Arg {argName = "converter", argType = TInterface "Gio" "Converter", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "ConverterOutputStream"
-- throws : False
-- Skip return : False

foreign import ccall "g_converter_output_stream_new" g_converter_output_stream_new :: 
    Ptr OutputStream ->                     -- base_stream : TInterface "Gio" "OutputStream"
    Ptr Converter ->                        -- converter : TInterface "Gio" "Converter"
    IO (Ptr ConverterOutputStream)


converterOutputStreamNew ::
    (MonadIO m, OutputStreamK a, ConverterK b) =>
    a ->                                    -- base_stream
    b ->                                    -- converter
    m ConverterOutputStream
converterOutputStreamNew base_stream converter = liftIO $ do
    let base_stream' = unsafeManagedPtrCastPtr base_stream
    let converter' = unsafeManagedPtrCastPtr converter
    result <- g_converter_output_stream_new base_stream' converter'
    checkUnexpectedReturnNULL "g_converter_output_stream_new" result
    result' <- (wrapObject ConverterOutputStream) result
    touchManagedPtr base_stream
    touchManagedPtr converter
    return result'

-- method ConverterOutputStream::get_converter
-- method type : OrdinaryMethod
-- Args : [Arg {argName = "_obj", argType = TInterface "Gio" "ConverterOutputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "_obj", argType = TInterface "Gio" "ConverterOutputStream", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "Converter"
-- throws : False
-- Skip return : False

foreign import ccall "g_converter_output_stream_get_converter" g_converter_output_stream_get_converter :: 
    Ptr ConverterOutputStream ->            -- _obj : TInterface "Gio" "ConverterOutputStream"
    IO (Ptr Converter)


converterOutputStreamGetConverter ::
    (MonadIO m, ConverterOutputStreamK a) =>
    a ->                                    -- _obj
    m Converter
converterOutputStreamGetConverter _obj = liftIO $ do
    let _obj' = unsafeManagedPtrCastPtr _obj
    result <- g_converter_output_stream_get_converter _obj'
    checkUnexpectedReturnNULL "g_converter_output_stream_get_converter" result
    result' <- (newObject Converter) result
    touchManagedPtr _obj
    return result'


