
{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)
-}

module GI.Gio.Objects.DBusObjectProxy
    ( 

-- * Exported types
    DBusObjectProxy(..)                     ,
    DBusObjectProxyK                        ,
    toDBusObjectProxy                       ,
    noDBusObjectProxy                       ,


 -- * Methods
-- ** dBusObjectProxyGetConnection
    dBusObjectProxyGetConnection            ,


-- ** dBusObjectProxyNew
    dBusObjectProxyNew                      ,




 -- * Properties
-- ** GConnection
    DBusObjectProxyGConnectionPropertyInfo  ,
    constructDBusObjectProxyGConnection     ,
    getDBusObjectProxyGConnection           ,


-- ** GObjectPath
    DBusObjectProxyGObjectPathPropertyInfo  ,
    constructDBusObjectProxyGObjectPath     ,
    getDBusObjectProxyGObjectPath           ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gio.Types
import GI.Gio.Callbacks
import qualified GI.GObject as GObject

newtype DBusObjectProxy = DBusObjectProxy (ForeignPtr DBusObjectProxy)
foreign import ccall "g_dbus_object_proxy_get_type"
    c_g_dbus_object_proxy_get_type :: IO GType

type instance ParentTypes DBusObjectProxy = DBusObjectProxyParentTypes
type DBusObjectProxyParentTypes = '[GObject.Object, DBusObject]

instance GObject DBusObjectProxy where
    gobjectIsInitiallyUnowned _ = False
    gobjectType _ = c_g_dbus_object_proxy_get_type
    

class GObject o => DBusObjectProxyK o
instance (GObject o, IsDescendantOf DBusObjectProxy o) => DBusObjectProxyK o

toDBusObjectProxy :: DBusObjectProxyK o => o -> IO DBusObjectProxy
toDBusObjectProxy = unsafeCastTo DBusObjectProxy

noDBusObjectProxy :: Maybe DBusObjectProxy
noDBusObjectProxy = Nothing

-- VVV Prop "g-connection"
   -- Type: TInterface "Gio" "DBusConnection"
   -- Flags: [PropertyReadable,PropertyWritable,PropertyConstructOnly]

getDBusObjectProxyGConnection :: (MonadIO m, DBusObjectProxyK o) => o -> m DBusConnection
getDBusObjectProxyGConnection obj = liftIO $ getObjectPropertyObject obj "g-connection" DBusConnection

constructDBusObjectProxyGConnection :: (DBusConnectionK a) => a -> IO ([Char], GValue)
constructDBusObjectProxyGConnection val = constructObjectPropertyObject "g-connection" val

data DBusObjectProxyGConnectionPropertyInfo
instance AttrInfo DBusObjectProxyGConnectionPropertyInfo where
    type AttrAllowedOps DBusObjectProxyGConnectionPropertyInfo = '[ 'AttrConstruct, 'AttrGet]
    type AttrSetTypeConstraint DBusObjectProxyGConnectionPropertyInfo = DBusConnectionK
    type AttrBaseTypeConstraint DBusObjectProxyGConnectionPropertyInfo = DBusObjectProxyK
    type AttrGetType DBusObjectProxyGConnectionPropertyInfo = DBusConnection
    type AttrLabel DBusObjectProxyGConnectionPropertyInfo = "DBusObjectProxy::g-connection"
    attrGet _ = getDBusObjectProxyGConnection
    attrSet _ = undefined
    attrConstruct _ = constructDBusObjectProxyGConnection

-- VVV Prop "g-object-path"
   -- Type: TBasicType TUTF8
   -- Flags: [PropertyReadable,PropertyWritable,PropertyConstructOnly]

getDBusObjectProxyGObjectPath :: (MonadIO m, DBusObjectProxyK o) => o -> m T.Text
getDBusObjectProxyGObjectPath obj = liftIO $ getObjectPropertyString obj "g-object-path"

constructDBusObjectProxyGObjectPath :: T.Text -> IO ([Char], GValue)
constructDBusObjectProxyGObjectPath val = constructObjectPropertyString "g-object-path" val

data DBusObjectProxyGObjectPathPropertyInfo
instance AttrInfo DBusObjectProxyGObjectPathPropertyInfo where
    type AttrAllowedOps DBusObjectProxyGObjectPathPropertyInfo = '[ 'AttrConstruct, 'AttrGet]
    type AttrSetTypeConstraint DBusObjectProxyGObjectPathPropertyInfo = (~) T.Text
    type AttrBaseTypeConstraint DBusObjectProxyGObjectPathPropertyInfo = DBusObjectProxyK
    type AttrGetType DBusObjectProxyGObjectPathPropertyInfo = T.Text
    type AttrLabel DBusObjectProxyGObjectPathPropertyInfo = "DBusObjectProxy::g-object-path"
    attrGet _ = getDBusObjectProxyGObjectPath
    attrSet _ = undefined
    attrConstruct _ = constructDBusObjectProxyGObjectPath

type instance AttributeList DBusObjectProxy = DBusObjectProxyAttributeList
type DBusObjectProxyAttributeList = ('[ '("g-connection", DBusObjectProxyGConnectionPropertyInfo), '("g-object-path", DBusObjectProxyGObjectPathPropertyInfo)] :: [(Symbol, *)])

type instance SignalList DBusObjectProxy = DBusObjectProxySignalList
type DBusObjectProxySignalList = ('[ '("interface-added", DBusObjectInterfaceAddedSignalInfo), '("interface-removed", DBusObjectInterfaceRemovedSignalInfo), '("notify", GObject.ObjectNotifySignalInfo), '("notify::[property]", GObjectNotifySignalInfo)] :: [(Symbol, *)])

-- method DBusObjectProxy::new
-- method type : Constructor
-- Args : [Arg {argName = "connection", argType = TInterface "Gio" "DBusConnection", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing},Arg {argName = "object_path", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "connection", argType = TInterface "Gio" "DBusConnection", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing},Arg {argName = "object_path", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "DBusObjectProxy"
-- throws : False
-- Skip return : False

foreign import ccall "g_dbus_object_proxy_new" g_dbus_object_proxy_new :: 
    Ptr DBusConnection ->                   -- connection : TInterface "Gio" "DBusConnection"
    CString ->                              -- object_path : TBasicType TUTF8
    IO (Ptr DBusObjectProxy)


dBusObjectProxyNew ::
    (MonadIO m, DBusConnectionK a) =>
    a ->                                    -- connection
    T.Text ->                               -- object_path
    m DBusObjectProxy
dBusObjectProxyNew connection object_path = liftIO $ do
    let connection' = unsafeManagedPtrCastPtr connection
    object_path' <- textToCString object_path
    result <- g_dbus_object_proxy_new connection' object_path'
    checkUnexpectedReturnNULL "g_dbus_object_proxy_new" result
    result' <- (wrapObject DBusObjectProxy) result
    touchManagedPtr connection
    freeMem object_path'
    return result'

-- method DBusObjectProxy::get_connection
-- method type : OrdinaryMethod
-- Args : [Arg {argName = "_obj", argType = TInterface "Gio" "DBusObjectProxy", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "_obj", argType = TInterface "Gio" "DBusObjectProxy", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "DBusConnection"
-- throws : False
-- Skip return : False

foreign import ccall "g_dbus_object_proxy_get_connection" g_dbus_object_proxy_get_connection :: 
    Ptr DBusObjectProxy ->                  -- _obj : TInterface "Gio" "DBusObjectProxy"
    IO (Ptr DBusConnection)


dBusObjectProxyGetConnection ::
    (MonadIO m, DBusObjectProxyK a) =>
    a ->                                    -- _obj
    m DBusConnection
dBusObjectProxyGetConnection _obj = liftIO $ do
    let _obj' = unsafeManagedPtrCastPtr _obj
    result <- g_dbus_object_proxy_get_connection _obj'
    checkUnexpectedReturnNULL "g_dbus_object_proxy_get_connection" result
    result' <- (newObject DBusConnection) result
    touchManagedPtr _obj
    return result'


