
{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

Information about a D-Bus property on a D-Bus interface.
-}

module GI.Gio.Structs.DBusPropertyInfo
    ( 

-- * Exported types
    DBusPropertyInfo(..)                    ,
    noDBusPropertyInfo                      ,


 -- * Methods
-- ** dBusPropertyInfoRef
    dBusPropertyInfoRef                     ,


-- ** dBusPropertyInfoUnref
    dBusPropertyInfoUnref                   ,




 -- * Properties
-- ** Annotations
    dBusPropertyInfoReadAnnotations         ,


-- ** Flags
    dBusPropertyInfoReadFlags               ,


-- ** Name
    dBusPropertyInfoReadName                ,


-- ** RefCount
    dBusPropertyInfoReadRefCount            ,


-- ** Signature
    dBusPropertyInfoReadSignature           ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gio.Types
import GI.Gio.Callbacks

newtype DBusPropertyInfo = DBusPropertyInfo (ForeignPtr DBusPropertyInfo)
foreign import ccall "g_dbus_property_info_get_type" c_g_dbus_property_info_get_type :: 
    IO GType

instance BoxedObject DBusPropertyInfo where
    boxedType _ = c_g_dbus_property_info_get_type

noDBusPropertyInfo :: Maybe DBusPropertyInfo
noDBusPropertyInfo = Nothing

dBusPropertyInfoReadRefCount :: DBusPropertyInfo -> IO Int32
dBusPropertyInfoReadRefCount s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Int32
    return val

dBusPropertyInfoReadName :: DBusPropertyInfo -> IO T.Text
dBusPropertyInfoReadName s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO CString
    val' <- cstringToText val
    return val'

dBusPropertyInfoReadSignature :: DBusPropertyInfo -> IO T.Text
dBusPropertyInfoReadSignature s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO CString
    val' <- cstringToText val
    return val'

dBusPropertyInfoReadFlags :: DBusPropertyInfo -> IO [DBusPropertyInfoFlags]
dBusPropertyInfoReadFlags s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 24) :: IO CUInt
    let val' = wordToGFlags val
    return val'

dBusPropertyInfoReadAnnotations :: DBusPropertyInfo -> IO [DBusAnnotationInfo]
dBusPropertyInfoReadAnnotations s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 32) :: IO (Ptr (Ptr DBusAnnotationInfo))
    val' <- unpackZeroTerminatedPtrArray val
    val'' <- mapM (newBoxed DBusAnnotationInfo) val'
    return val''

-- method DBusPropertyInfo::ref
-- method type : OrdinaryMethod
-- Args : [Arg {argName = "_obj", argType = TInterface "Gio" "DBusPropertyInfo", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "_obj", argType = TInterface "Gio" "DBusPropertyInfo", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TInterface "Gio" "DBusPropertyInfo"
-- throws : False
-- Skip return : False

foreign import ccall "g_dbus_property_info_ref" g_dbus_property_info_ref :: 
    Ptr DBusPropertyInfo ->                 -- _obj : TInterface "Gio" "DBusPropertyInfo"
    IO (Ptr DBusPropertyInfo)


dBusPropertyInfoRef ::
    (MonadIO m) =>
    DBusPropertyInfo ->                     -- _obj
    m DBusPropertyInfo
dBusPropertyInfoRef _obj = liftIO $ do
    let _obj' = unsafeManagedPtrGetPtr _obj
    result <- g_dbus_property_info_ref _obj'
    checkUnexpectedReturnNULL "g_dbus_property_info_ref" result
    result' <- (wrapBoxed DBusPropertyInfo) result
    touchManagedPtr _obj
    return result'

-- method DBusPropertyInfo::unref
-- method type : OrdinaryMethod
-- Args : [Arg {argName = "_obj", argType = TInterface "Gio" "DBusPropertyInfo", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- Lengths : []
-- hInArgs : [Arg {argName = "_obj", argType = TInterface "Gio" "DBusPropertyInfo", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, transfer = TransferNothing}]
-- returnType : TBasicType TVoid
-- throws : False
-- Skip return : False

foreign import ccall "g_dbus_property_info_unref" g_dbus_property_info_unref :: 
    Ptr DBusPropertyInfo ->                 -- _obj : TInterface "Gio" "DBusPropertyInfo"
    IO ()


dBusPropertyInfoUnref ::
    (MonadIO m) =>
    DBusPropertyInfo ->                     -- _obj
    m ()
dBusPropertyInfoUnref _obj = liftIO $ do
    let _obj' = unsafeManagedPtrGetPtr _obj
    g_dbus_property_info_unref _obj'
    touchManagedPtr _obj
    return ()


